#include "Wire.h"

//
// Utiliza dos puentes de resistencias como estan explicados mas abajo
// para leer la temperatura y el voltaje de la bateria
// Ademas utiliza el modulo Grove MPU6050 Acelerometer /Gyroscope & Compass
// I2Cdev and MPU6050 must be installed as libraries, or else the .cpp/.h files
// for both classes must be in the include path of your project
#include "I2Cdev.h"
#include "MPU6050.h"
#include <math.h>

// class default I2C address is 0x68
// specific I2C addresses may be passed as a parameter here
// AD0 low = 0x68 (default for InvenSense evaluation board)
// AD0 high = 0x69

// Pin conectado al sensor de temperatura
#define PINTEM A2
// Pin conectado al positivo de bateria
#define PINBATT A0
MPU6050 accelgyro;

int16_t ax, ay, az;
int16_t gx, gy, gz;
int16_t mx, my, mz;

float heading;
float tiltheading;

float Axyz[3];
float Gxyz[3];
float Mxyz[3];
float sumAxyz[3];
float sumGxyz[3];
float sumMxyz[3];
float bucleCounter = 15.0; // NUmero de veces que se lee el sensor para obtener el valor medio

#define LED_PIN 13

void setup() {
  // join I2C bus (I2Cdev library doesn't do this automatically)
  Wire.begin();

  // initialize serial communication
  // (38400 chosen because it works as well at 8MHz as it does at 16MHz, but
  // it's really up to you depending on your project)
  Serial.begin(9600);

  // initialize device
  Serial.println("Initializing I2C devices...");
  accelgyro.initialize();

  // verify connection
  Serial.println("Testing device connections...");
  Serial.println(accelgyro.testConnection() ? "MPU6050 connection successful" : "MPU6050 connection failed");

  // configure Arduino LED for
  pinMode(LED_PIN, OUTPUT);
}
void loop() 
{
   String Out;
  // blink LED to indicate test activity
  digitalWrite(LED_PIN, 1);
  Out = ReadDataLine();
  digitalWrite(LED_PIN, 0);
  Serial.println(Out);
  delay(1000);
  statusDataLine();

}

void statusDataLine() 
{
 int i;
 for(i=0;i< bucleCounter ;i++)
  // read raw accel/gyro measurements from device
  {
  accelgyro.getMotion9(&ax, &ay, &az, &gx, &gy, &gz, &mx, &my, &mz);
  getAccelValue();
  getGyroValue();
  getCompassValue();
  getHeading();
  getTiltHeading();    // these methods (and a few others) are also available
  //accelgyro.getAcceleration(&ax, &ay, &az);
  //accelgyro.getRotation(&gx, &gy, &gz);
  // display tab-separated accel/gyro x/y/z values
  
  }
  
  for(i=0;i<3;i++)
  {
    
    Axyz[i]=sumAxyz[i]/bucleCounter;
    Gxyz[i]=sumGxyz[i]/bucleCounter;
    Mxyz[i]=sumMxyz[i]/bucleCounter;
    sumAxyz[i] =0;
    sumGxyz[i]=0;
    sumMxyz[i]=0;
  }
  Serial.println("************************************");
  Serial.print("BaAtt: ");
  Serial.println( voltimeter(PINBATT)) ; // Display Volt Batt
  Serial.print("Temp: ");
  Serial.println(Thermistor(analogRead(PINTEM)));  // display Fahrenheit   
  Serial.print("Acceleration(g) of X,Y,Z: ");
  Serial.print(Axyz[0]);
  Serial.print(",");
  Serial.print(Axyz[1]);
  Serial.print(",");
  Serial.println(Axyz[2]);
  Serial.print("Gyro(degress/s) of X,Y,Z:");
  Serial.print(Gxyz[0]);
  Serial.print(",");
  Serial.print(Gxyz[1]);
  Serial.print(",");
  Serial.println(Gxyz[2]);
  Serial.print("Compass Value of X,Y,Z:");
  Serial.print(Mxyz[0]);
  Serial.print(",");
  Serial.print(Mxyz[1]);
  Serial.print(",");
  Serial.println(Mxyz[2]);
  Serial.print("Clockwise angle between the magnetic north and X-Axis:");
  Serial.println(heading);
  Serial.print("Clockwise angle between the magnetic north and the projection of the positive X-Axis in the horizontal plane:");
  Serial.println(tiltheading);
  Serial.println("************************************");
}

//
// Esta funcion lee todos los valores del estado del robot
// Lee y almacena en dataLine los datos de posicion y estado del KM82
// 
String ReadDataLine(){
 String dataLine;
 int i;
 for(i=0;i< bucleCounter ;i++)
  // read raw accel/gyro measurements from device
  {
  accelgyro.getMotion9(&ax, &ay, &az, &gx, &gy, &gz, &mx, &my, &mz);
  getAccelValue();
  getGyroValue();
  getCompassValue();
  getHeading();
  getTiltHeading();    // these methods (and a few others) are also available
  //accelgyro.getAcceleration(&ax, &ay, &az);
  //accelgyro.getRotation(&gx, &gy, &gz);
  // display tab-separated accel/gyro x/y/z values
  
  }
  
  for(i=0;i<3;i++)
  {  
    Axyz[i]=sumAxyz[i]/bucleCounter;
    Gxyz[i]=sumGxyz[i]/bucleCounter;
    Mxyz[i]=sumMxyz[i]/bucleCounter;
    sumAxyz[i] =0;
    sumGxyz[i]=0;
    sumMxyz[i]=0;
  }
  dataLine = "";
  dataLine = dataLine + voltimeter(PINBATT); // Display Volt Batt
  dataLine = dataLine + "|";
  dataLine = dataLine + Thermistor(analogRead(PINTEM));  // display Temp Celsius   
  dataLine = dataLine + "|";
  dataLine = dataLine + Axyz[0];
  dataLine = dataLine + "|";
  dataLine = dataLine + Axyz[1];
  dataLine = dataLine + "|";
  dataLine = dataLine + Axyz[2];
  dataLine = dataLine + "|";
  dataLine = dataLine + Gxyz[0];
  dataLine = dataLine + "|";
  dataLine = dataLine + Gxyz[1];
  dataLine = dataLine + "|";
  dataLine = dataLine + Gxyz[2];
  dataLine = dataLine + "|";
  dataLine = dataLine + Mxyz[0]; 
  dataLine = dataLine + "|";
  dataLine = dataLine + Mxyz[1];
  dataLine = dataLine + "|";
  dataLine = dataLine + Mxyz[2];
  dataLine = dataLine + "|";
  dataLine = dataLine + heading;
  dataLine = dataLine + "|";
  dataLine = dataLine + tiltheading;
  // Serial.println("************************************");
  // Serial.println(dataLine);
  // Serial.println("************************************");
  return  dataLine;
}

double voltimeter(int batPin)
//
// Mide valor del voltimetro MAX 15V
// +Vin ----[R1 22K]-- A1 ---[R2 10K]-- GROUND
//

{
float val = analogRead(batPin);
double volt = val*15/1007;  // Maps Readings For zero to 15 displayed values.
return volt;
}

double Thermistor(int RawADC) {
 double Temp;
 Temp = log(10000.0*((1024.0/RawADC-1))); 
//         =log(10000.0/(1024.0/RawADC-1)) // for pull-up configuration
 Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp ))* Temp );
 Temp = Temp - 273.15;            // Convert Kelvin to Celcius
 // ThermistorThermistorThermistorTemp = (Temp * 9.0)/ 5.0 + 32.0; // Convert Celcius to Fahrenheit
 return Temp;
}

void getAccelValue(void)
{
  Axyz[0] = (double) ax / 16384;
  Axyz[1] = (double) ay / 16384;
  Axyz[2] = (double) az / 16384; 
  sumAxyz[0] += Axyz[0];
  sumAxyz[1] += Axyz[1];
  sumAxyz[2] += Axyz[2];
  
}
void getGyroValue(void)
{
  Gxyz[0] = (double) gx * 250 / 32768;
  Gxyz[1] = (double) gy * 250 / 32768;
  Gxyz[2] = (double) gz * 250 / 32768;
  sumGxyz[0] += Gxyz[0];
  sumGxyz[1] += Gxyz[1];
  sumGxyz[2] += Gxyz[2];
}
void getCompassValue(void)
{
  Mxyz[0] = (double) mx * 1200 / 4096;
  Mxyz[1] = (double) my * 1200 / 4096;
  Mxyz[2] = (double) mz * 1200 / 4096;
  sumMxyz[0] += Mxyz[0];
  sumMxyz[1] += Mxyz[1];
  sumMxyz[2] += Mxyz[2];
}
void getHeading(void)
{
  heading=180*atan2(Mxyz[1],Mxyz[0])/PI;
  if(heading <0) heading +=360;
}

void getTiltHeading(void)
{
  float pitch = asin(-Axyz[0]);
  float roll = asin(Axyz[1]/cos(pitch));

  float xh = Mxyz[0] * cos(pitch) + Mxyz[2] * sin(pitch);
  float yh = Mxyz[0] * sin(roll) * sin(pitch) + Mxyz[1] * cos(roll) - Mxyz[2] * sin(roll) * cos(pitch);
  float zh = -Mxyz[0] * cos(roll) * sin(pitch) + Mxyz[1] * sin(roll) + Mxyz[2] * cos(roll) * cos(pitch);
  tiltheading = 180 * atan2(yh, xh)/PI;
  if(yh<0)    tiltheading +=360;
}
