// Kool Mechatronics KM82 library
// copyright Open Hardware SAC, 2014
// this code is public domain, enjoy!

#include <AFMotor.h>
#include <Ultrasonic.h>
#include <Servo.h>

// Line Follow L (A0) C (A1) R(D9)
#define pinFollowL 14
#define pinFollowC 15
#define pinFollowR 9
#define pinServo 10

#define ___  = 0
#define L__  = 1
#define _C_  = 2
#define __R  = 4
#define LC_  = 3
#define _CR  = 6
#define L_R  = 5
#define LCR  = 7



int maxDistance = 200 * 58; // Maxima distancia de medida por el sensor ultrasonico
int debugMonitor = 1;  // Debug info with serial.print
int crash = 35; // Cm to chrash
int chassisSpeed = 120; // Velocidad estandar de los motores.
int turnSpeed = 250;  // Velocidad standar de giro
int mode = 0; // robot scaner 1 seguidor de linea

AF_DCMotor motorL(2);
AF_DCMotor motorR(1);
Servo myservo;  // create servo object to control a servo 

Ultrasonic ultraBack(19,18,maxDistance);   // (Trig PIN,Echo PIN)
Ultrasonic ultraFront(17,16,maxDistance);  // (Trig PIN,Echo PIN)

void setup() {
  Serial.begin(9600);           // set up Serial library at 9600 bps
  pinMode (pinFollowL,INPUT);
  pinMode (pinFollowC,INPUT);
  pinMode (pinFollowR,INPUT);
  
  Serial.println("KM82 chassis start!");
  myservo.attach(10);
  myservo.write(89);  

  // turn on motor
  motorL.setSpeed(chassisSpeed);
  motorR.setSpeed(chassisSpeed);
 
  motorL.run(RELEASE);
  motorR.run(RELEASE);
  mode =  readLineSensors();
  randomSeed(analogRead(0));
}

void loop() {
  // demo();
   if (mode != 4) 
     goMode();
   else 
     followLine();
  // chassisRELEASE();
  // delay(500);
}

 /**************************** Chassis & Motors functions *****************************/

void chassisForward(int speedL, int speedR){
    if (debugMonitor){
      Serial.print ("Forward -> L:"); Serial.print (speedL);
      Serial.print (" R:");Serial.println (speedR);
    }      
    motorL.run(FORWARD);
    motorR.run(FORWARD);
    motorL.setSpeed(speedL);  
    motorR.setSpeed(speedR);      
  }

void chassisBackward(int speedL, int speedR){
    if (debugMonitor){
      Serial.print ("Backward -> L:"); Serial.print (speedL);
      Serial.print (" R:"); Serial.println (speedR);
    }
    motorL.run(BACKWARD);
    motorR.run(BACKWARD);
    motorL.setSpeed(speedL);  
    motorR.setSpeed(speedR);      
  }
  
void chassisTurnL(int speedT){
    if (debugMonitor){
      Serial.print ("Turn L ->"); Serial.println (speedT);
    }
    motorL.run(BACKWARD);
    motorR.run(FORWARD);
    motorL.setSpeed(speedT);  
    motorR.setSpeed(speedT);      
  }
bool chassisTurnLtolDistance(int speedT){ 
  // gira hasta que encuentra un camino
  // return TRUE if scape is found
  uint8_t i;
  for (i=1; i< 100; i+=5) { // 2 sec aprox de giro
    if (distance(0,90) < crash) 
      chassisTurnL(250);  
    else {
        delay(10); // corrije la rotacion.
        chassisRELEASE(); 
        return true;
        break;         
        }   
    delay(20);
   }
   return false;
}
  
void chassisTurnR(int speedT){
    if (debugMonitor){
      Serial.print ("Turn R ->"); Serial.println (speedT);
    }
    motorL.run(FORWARD);
    motorR.run(BACKWARD);
    motorL.setSpeed(speedT);  
    motorR.setSpeed(speedT);      
  }

void chassisRELEASE(){
    if (debugMonitor){
      Serial.println ("Stop");
    }
    motorL.run(RELEASE);
    motorR.run(RELEASE);
  }
bool chassisTurnRtolDistance(int speedT){ 
  // gira hasta que encuentra un camino
  // return TRUE if scape is found
  uint8_t i;
  for (i=1; i< 100; i+=5) { // 2 sec aprox de giro
    if (distance(0,90) < crash) 
      chassisTurnR(250);  
    else {
        delay(10); // corrije la rotacion.
        chassisRELEASE(); 
        return true;
        break;         
        }   
    delay(20);
   }
   return false;
}
 
 /**************************** Ultrasonic functions ***********************************/
int distance(int sensor, int angle){ // Sensor 0 front, sensor 1 back, angle servo pos
  int result = 0;
  if (!sensor) {
    result = ultraFront.Ranging(CM); // CM or INC
  Serial.print("Front: ");
  Serial.print(result); // CM or INC
  Serial.println(" cm     " );
  delay(50);
  }
  else {
     result = ultraBack.Ranging(CM); // CM or INC 
  Serial.print("Back: ");
  Serial.print(result); // CM or INC
  Serial.println(" cm" );
  delay(50);
  }
  return result;
}
/********************** Follow line funcions ********************/

int readLineSensors(){
  int readed = 0;
  readed = !digitalRead(pinFollowC) * 1    ; // _C_
  readed += ( !digitalRead(pinFollowL) * 2 ); // L__
  readed += ( !digitalRead(pinFollowR) * 4 ); // __R
return readed;
}
/************************** Top Functions **********************/

void demo(){
  // chassisForward( 150,150 );
  chassisTurnR(220);
}

void goMode(){
  // 
  // Avanza esquivando los obstaculos y buscando salidas.
  // Utiliza sensor de distancia delantero, trasero y servo.
  // Velocidad estandar chassisSpeed
  //

  uint8_t i = 200;
  
  chassisForward( chassisSpeed,chassisSpeed );
  if (distance(0,90) < crash) {
     switch (random(2)) {
       case 0 : {
             Serial.println("Turn L" );
             if (!chassisTurnLtolDistance(255)) {
              // No ha encontrado camino y retrocede
              for (i=100; i<255; i+=5) {
                if (distance(1,90) > crash) 
                  chassisBackward(chassisSpeed,chassisSpeed);  
                else {
                    chassisRELEASE(); 
                    break;         
                    }   
                delay(20);
               }
            }
            break;
           }
       case 1 : {
             Serial.println("Turn RL" );
               if (!chassisTurnRtolDistance(255)) {
              // No ha encontrado camino y retrocede
              for (i=100; i<255; i+=5) {
                if (distance(1,90) > crash) 
                  chassisBackward(chassisSpeed,chassisSpeed);  
                else {
                    chassisRELEASE(); 
                    break;         
                    }   
                delay(20);
               }
            }
            break;
           }
      
     }
  } 
}

int followLine(){
  
//
// En modo seguidor de linea el chassis funciona en reversa.
  int readed;
  readed = readLineSensors();
  // Serial.print ( readed ); 
  switch (readed){
     case 0 : {
              Serial.println( "___ stop" );
              chassisRELEASE();
              break;
               }
     case 3 : {
              Serial.println( "L__ tuerce a la derecha" );
              // chassisRELEASE();
              chassisTurnL(turnSpeed);
             break;
     }
     case 1 : {
              Serial.println( "LC_ tuerce un poco a la derecha" );
              // chassisRELEASE();
              chassisTurnL(turnSpeed);
              break;
     }
     case 2 : {
              Serial.println( "_C_ es imposible" );
              chassisRELEASE();
              break;
     }
     case 6 : {
              Serial.println( "__R tuerce a la izquierda" );
              // chassisRELEASE();
              chassisTurnR(turnSpeed);
              break;
     }
     case 4 : {
              Serial.println( "_CR tuerce un poco a la izquierda" );
              // chassisRELEASE();
              chassisTurnR(turnSpeed);
             break;
     }
     case 5 : {
              Serial.println( "L_R sigue Recto..." );
              chassisBackward( chassisSpeed, chassisSpeed );
              break;
     }
     case 7 : {
              Serial.println( "LCR Se ha salido de la linea");
              // chassisRELEASE();
              break;
     }
  }
}
