/*
  km82Chassis.cpp - Library for KM82 motor Chassis
  Released into the public domain.
  by Fernando Landaluce
  june 2013
*/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
*/

// km82Chassis
// KM82 Chassis Motor controler
//

// Version 0.01 ALPHA Experimental version

// Controla los motores de traccion del chasis principal de km82
// se controla la velocidad, el sentido, el recorrido efectuado por el motor
// permite moverlo X centimetros en cualquier direccion
// 
 

#include "Arduino.h"
#include "km82Motor.h" 
#include "km82Chassis.h"

km82Chassis::km82Chassis(int motor_L_pin_1, int motor_L_pin_2, int motorEnablePWM_L, int motor_R_pin_1, int motor_R_pin_2, int motorEnablePWM_R )
{
turnSpeed = 220; // Velocidad que se usa en los motores para los giros
motorLeft = new km82Motor(motor_L_pin_1, motor_L_pin_2, motorEnablePWM_L);
motorRigth = new km82Motor(motor_R_pin_1, motor_R_pin_2, motorEnablePWM_R);

}

void km82Chassis::test(){
  this->reset();
  this->On();
  this->run(5);
  this->setSpeed(200);
  /*
  this->run(500);
  this->setSpeed(250);
  delay(500);
  this->setSpeed(190);  
  this->turnLeft(1100);
  
  this->forward(500);

  this->turnRight(1100);
  
  this->forward(1000);

  // this->motorRigth->reverse();
  
  this->brake(200);
  this->setSpeed(250);  
  this->turnRight(2000);
  this->turnLeft(2000);
  this->brake(200);
  this->backward(500);
  */
  int pulseTime = 100 ;
  for(int x = 0; x <= 30; x++){
	  this->turnRight(pulseTime);
	  this->brake();
	  //delay(100);
  }
  for(int x = 0; x <= 30; x++){
	  this->turnLeft(pulseTime);
	  this->brake();
	  //delay(100);
  }
  this->Off();
  delay(2000);
}
    				 
void km82Chassis::On()
{
  this->motorLeft->On();
  this->motorRigth->On();
}
 
void km82Chassis::Off()
{
  this->motorLeft->Off();
  this->motorRigth->Off();
}
 
void km82Chassis::coast()
{
  this->motorLeft->Coast();
  this->motorRigth->Coast();
}
 
// void km82Chassis::brake(int time)
void km82Chassis::brake()
{
  this->motorLeft->Brake();
  this->motorRigth->Brake();
  // delay(time);
  // this->Off(); // Apaga el motor
}

void km82Chassis::run(int time)
{
  this->motorLeft->Run();
  this->motorRigth->Run();
 delay(time);
}

void km82Chassis::setSpeed(int speed)
{
  this->motorLeft->setSpeed(speed);
  this->motorRigth->setSpeed(speed);
}

void km82Chassis::setSpeed(int speedL, int speedR)
{
  this->motorLeft->setSpeed(speedL);
  this->motorRigth->setSpeed(speedR);
}

void km82Chassis::reset()
{
  this->motorLeft->Reset();
  this->motorRigth->Reset();
}

void km82Chassis::forward(int time)
{
  this->motorLeft->setDirection(FORWARD);
  this->motorRigth->setDirection(FORWARD);
  if (time > 0) delay(time);
}
 
void km82Chassis::backward(int time)
{
  this->motorLeft->setDirection(BACKWARD);
  this->motorRigth->setDirection(BACKWARD);
  if (time > 0) delay(time);
}

void km82Chassis::turnLeft(int time)
{
	this->turnLeft(time, 0);
}
 
void km82Chassis::turnLeft(int time, int brakeTime)
{
  // Gira el chasis , 
  // guarda la velocidadactual, la sube al maximo
  // cumple el tiempo de giro y recupera la velocidad anterior.
  int speedL = this->motorLeft->getSpeed();
  int speedR = this->motorRigth->getSpeed();
  int dirL = this->motorLeft->getDirection();
  int dirR = this->motorRigth->getDirection();
  int runL = this->motorLeft->runnig();
  int runR = this->motorRigth->runnig();

  this->motorLeft->setSpeed(this->turnSpeed);
  this->motorRigth->setSpeed(this->turnSpeed);
  this->motorLeft->setDirection(BACKWARD);
  this->motorRigth->setDirection(FORWARD);
  delay(time);
 if (brakeTime > 0){
  	this->motorLeft->Brake();
  	this->motorRigth->Brake();
  	delay(brakeTime);
 }

  // recupera el estado la velocidad y direccion 
  this->motorLeft->setDirection(dirL);
  this->motorRigth->setDirection(dirR);
  this->motorLeft->setSpeed(speedL);
  this->motorRigth->setSpeed(speedR);
  if(runL) this->motorLeft->On();
  if(runR) this->motorRigth->On();
}


void km82Chassis::turnLeftAngle(int angle)
{
}
 
void km82Chassis::turnRight(int time)
{
	this->turnRight(time,0);
}

void km82Chassis::turnRight(int time, int brakeTime)
{
  // Gira el chasis , 
  // guarda la direccion, la velocidad actual, la sube al maximo
  // cumple el tiempo de giro y recupera la velocidad y direccion anterior.
  int speedL = this->motorLeft->getSpeed();
  int speedR = this->motorRigth->getSpeed();
  int dirL = this->motorLeft->getDirection();
  int dirR = this->motorRigth->getDirection();
  int runL = this->motorLeft->runnig();
  int runR = this->motorRigth->runnig();
  
  this->motorLeft->setSpeed(this->turnSpeed);
  this->motorRigth->setSpeed(this->turnSpeed);
  this->motorLeft->setDirection(FORWARD);
  this->motorRigth->setDirection(BACKWARD);
 delay(time);
 if (brakeTime > 0){
  	this->motorLeft->Brake();
  	this->motorRigth->Brake();
  	delay(brakeTime);
 }
  // recupera la velocidad y direccion 
  this->motorLeft->setDirection(dirL);
  this->motorRigth->setDirection(dirR);
  this->motorLeft->setSpeed(speedL);
  this->motorRigth->setSpeed(speedR);
  if(runL) this->motorLeft->On();
  if(runR) this->motorRigth->On();
}
 

void km82Chassis::turnLeft()
{
  this->motorLeft->setSpeed(this->turnSpeed);
  this->motorRigth->setSpeed(this->turnSpeed);
  this->motorLeft->setDirection(BACKWARD);
  this->motorRigth->setDirection(FORWARD);
}

void km82Chassis::turnRight(){
  this->motorLeft->setSpeed(this->turnSpeed);
  this->motorRigth->setSpeed(this->turnSpeed);
  this->motorLeft->setDirection(FORWARD);
  this->motorRigth->setDirection(BACKWARD);
}
