/*
  km82Motor.cpp - Library for KM82 motor 
  Released into the public domain.
  by Fernando Landaluce
  june 2013
*/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
*/

// km82Motor
// km82 motor controler
//

// Version 0.01 ALPHA Experimental version

// Controla el motor de traccion del chasis principal de km82
// se controla la velocidad, el sentido, el recorrido efectuado por el motor
// permite moverlo X centimetros en cualquier direccion
// 


#include "Arduino.h"
#include "km82Motor.h"
// #include "Wire.h"

// MODIFICAR PARA CAMBIAR LA VELOCIDAD CON NEGATIVOS  

km82Motor::km82Motor(int motor_pin_1, int motor_pin_2, int motorEnablePWM )
{

this->motorDebug = false;
this->debug = false;
this->motorPin1 = motor_pin_1;    
this->motorPin2 = motor_pin_2;  
this->motorEnablePWM = motorEnablePWM; 

// Indica si el controlador va a utilizar un motor derecho o izquierdo
// dependiendo del tipo se invierten las ordenes adelante atras, etc.. 
// por defecto el motor es izquierdo

this->motorOn = false;    // Motor ON OFF
this->motorDirection = FORWARD; // Forward TRUE  Reverse FALSE

// Nueva version la velocidad de los motores de de 1 a 10
// el calculo se hace
// speed = map(analogValue,1,10,speedMin,speedMax);  // convert the speed 0 - 255 to 1 to 10

this->motorMinSpeed = 0; // 1 a 255 valor mínimo para que el motor se mueva.
this->motorMaxSpeed = 255; // 1 a 255

this->motorDefaultSpeed = 5; // 1 a 10

this->motorSpeed = this->motorDefaultSpeed ; 

// Internal

this->INTTIME = 15624 / 800 ; // = (16*10^6) / (1*1024) - 1 (must be <65536) 1s = 15624

// setup the pins on the microcontroller:
  pinMode(this->motorEnablePWM, OUTPUT);
  pinMode(this->motorPin1, OUTPUT);
  pinMode(this->motorPin2, OUTPUT);
}



void km82Motor::On() // start motor
{
	if (!this->motorOn) {
	this->motorOn = true;
	if (this->debug) Serial.println("km82Motor::On()");
	this->setSpeed(this->motorSpeed);
    digitalWrite(this->motorPin1, LOW);
    digitalWrite(this->motorPin2, HIGH);
    // analogWrite(this->motorEnablePWM, this->motorSpeed);
    }
}

void km82Motor::Off() // stop motor
{
	if (this->motorOn) {
	this->motorOn = false;
	if (this->debug) Serial.println("km82Motor::Off()");
    analogWrite(this->motorEnablePWM, 0);
	digitalWrite(this->motorPin1, LOW);
	digitalWrite(this->motorPin2, LOW);
	}
}
void km82Motor::Coast() // Apaga el motor y lo libera
{
	this->Off();
	/*
	this->motorOn = false;
	if (this->debug) Serial.println("km82Motor::Coast()");
	digitalWrite(this->motorPin1, LOW);
	digitalWrite(this->motorPin2, LOW);
	*/
}

void km82Motor::Brake() // stop motor con freno el tiempo de frenado se cpontrola externamente
{
	this->motorOn = false; // Comprobar si es necesario a false o es mejor mantenerlo a on para continuar
	this->Off(); // apaga motor
	if (this->debug) Serial.println("km82Motor::Brake()");
	digitalWrite(this->motorPin1, HIGH);
    digitalWrite(this->motorPin2, HIGH);
}

void km82Motor::Reset() // Restea todo a 0
{
	this->Off();
	this->setDirection(FORWARD);
	this->motorSpeed = this->motorDefaultSpeed;
	if (this->debug) Serial.println("km82Motor::Reset()");
	
}

boolean km82Motor::runnig() // devuelve true si esta funcionando false esta apagado
{
	return this->motorOn;
}

void km82Motor::reversePower() // reverse motor encendido apagado
{
	if (this->debug) Serial.println("km82Motor::reversePower()");
	this->motorOn = !this->motorOn;
	if(!this->motorOn) 
		this->Off();
	else 
		this->On();
}

void km82Motor::reverse() // reverse sentido giro del motor
{
	if (this->debug) Serial.println("km82Motor::reverse()");
	this->setDirection(!this->motorDirection);
}


void km82Motor::setDirection(boolean directionChange){
	if (this->debug) Serial.println("km82Motor::setDirection()");
	switch(directionChange) {
		case FORWARD : {
						this->motorDirection = FORWARD;
						digitalWrite(this->motorPin1, LOW);
						digitalWrite(this->motorPin2, HIGH);
						break;
						}
					
			 default : {
			 			this->motorDirection = BACKWARD;
						digitalWrite(this->motorPin1, HIGH);
						digitalWrite(this->motorPin2, LOW);
						break;
						}
	}
}

boolean km82Motor::getDirection() // return curren Direction
{ 
	return this->motorDirection;
}

void km82Motor::setSpeed(int speed)  // set curren Direction
// Esta funcion es la que pone en marcha el motor a nivel mas basico (todas la llaman)
// Hay que modificarla para que admita negativos en la velocidad
{ 
	if (this->debug) Serial.println("km82Motor::setSpeed()");

	// speed = speed > 255 ? 255 : speed;
	// speed = speed < -255 ? -255 : speed;
	speed = speed > 10 ? 10 : speed;
	speed = speed < -10 ? -10 : speed;
	if (speed > 0) this->setDirection(FORWARD);
	if (speed < 0) 
		{
		this->setDirection(BACKWARD);
		speed *= -1; 
		}
    speed = map(speed,1,10,this->motorMinSpeed,this->motorMaxSpeed);  // convert the analog value

	this->motorSpeed = speed;
	if(this->motorDebug) { analogWrite(this->motorEnablePWM, 0);return; } // test mode motores siempre OFF
    analogWrite(this->motorEnablePWM, this->motorSpeed);
}


int  km82Motor::getSpeed() // read elocidad del motor
{ 
	return this->motorSpeed;
}

void km82Motor::setMinSpeed(int speed) // set motorMinSpeed 1 to 255 min value to start motor
{
	this->motorMinSpeed = speed;
}

void km82Motor::Run(){
	 if (this->debug) Serial.println("km82Motor::Run()");
     this->setDirection(this->motorDirection);
     this->On();
} 



void km82Motor::printStatus(){
  Serial.print("motorOn:");
  Serial.print(this->motorOn);
  Serial.print(" Direction:");
  Serial.print(this->motorDirection);
  Serial.print(" motorSpeed:");
  Serial.print(this->motorSpeed);
  Serial.println(" ");
}
