/*
  km82Motor.h - Library for km82 motors
  Released into the public domain.
  by Fernando Landaluce
  abril 2013
*/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
*/

#ifndef km82Motor_h
#define km82Motor_h

#include "Arduino.h"

const boolean FORWARD  = true;
const boolean BACKWARD = false;


class km82Motor
{
  public:
    km82Motor(int motor_pin_1, int motor_pin_2, int motorEnablePWM);
    void Run();
    void On();  // On motor
    void Off();  // stop motor
    void Coast(); // Libera el motor 
	void Brake(); // stop motor con freno
    void reversePower(); // reverse motor encendido apagado
    void reverse(); // reverse sentido giro del motor
    void setMinSpeed(int speed); // set motorMinSpeed 1 to 255 min value to start motor
    void setSpeed(int speed); // set Velocidad del motor
    int  getSpeed(); // read elocidad del motor
    boolean runnig(); // devuelve true si esta funcionando false esta apagado
    void Reset(); // Restea todo a 0
    void setDirection(boolean directionChange); // FORWARD / BACKWARD
    boolean getDirection();
    void printStatus();

  private:
    int motorPin1;    // pin 1 del motor
    int motorPin2;    // pin2 del motor
    int motorEnablePWM ;    // pin enable/disable PWM control motor
    int motorOn;    // Motor ON OFF
    int motorDirection; // Forward TRUE  Reverse FALSE
    int motorSpeed; // 1 a 10 10 maximo 1 minimo
	int motorMinSpeed; // 1 a 255 valor mínimo para que el motor se mueva.
	int motorMaxSpeed; // 1 a 255 valor limite para el pin;
    int motorDefaultSpeed; // 1 a 10 1 maximo 10 minimo
    int motorDebug; // si es true los motores no se mueven
    int debug; // nuestra valores por serial
	
// Internal
	int  INTTIME ;// = (16*10^6) / (1*1024) - 1 (must be <65536) 1s = 15624

};

#endif